<?php

namespace App\Dto\Input;

use App\Entity\OrganizationalUnit;
use App\Entity\User;
use App\Entity\UserGroup;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Validator\Constraints as Assert;

final class UserInput
{
    #[Assert\NotBlank]
    #[Groups(['user:write'])]
    public ?string $username = null;

    #[Groups(['user:write'])]
    public array $roles = [];

    /**
     * @var OrganizationalUnit[]
     */
    #[Groups(['user:write'])]
    public array $allowedOrganizationalUnits = [];

    #[Assert\NotBlank(groups: ['user:create'])]
    #[Assert\Length(min: 8)]
    #[Groups(['user:write'])]
    public ?string $password = null;

    #[Assert\NotNull]
    #[Groups(['user:write'])]
    public ?bool $enabled = true;

    /**
     * @var UserGroup[]
     */
    #[Groups(['user:write'])]
    public array $userGroups = [];

    public function __construct(?User $user = null)
    {
        if (!$user) {
            return;
        }

        $this->username = $user->getUsername();
        $this->roles = $user->getRoles();
        $this->enabled= $user->isEnabled();
        $this->userGroups = $user->getUserGroups()->toArray();
        $this->allowedOrganizationalUnits = $user->getAllowedOrganizationalUnits()->toArray();
    }

    public function createOrUpdateEntity(?User $user = null): User
    {
        if (!$user) {
            $user = new User();
        }

        $user->setUsername($this->username);
        $user->setRoles($this->roles);
        $user->setEnabled($this->enabled);
        $user->setUserGroups($this->userGroups);

        if ($this->password !== null) {
            $user->setPassword($this->password);
        }

        //$user->setAllowedOrganizationalUnits($this->allowedOrganizationalUnits);

        return $user;
    }
}