<?php

namespace Functional;

use App\Entity\User;
use App\Factory\UserFactory;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\DecodingExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

class UserTest extends AbstractTest
{
    CONST USER_ADMIN = 'ogadmin';
    CONST USER_CREATE = 'test-create';
    CONST USER_UPDATE = 'test-update';
    CONST USER_DELETE = 'test-delete';

    /**
     * @throws RedirectionExceptionInterface
     * @throws DecodingExceptionInterface
     * @throws ClientExceptionInterface
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     */
    public function testGetCollection(): void
    {
        UserFactory::createOne(['username' => self::USER_ADMIN]);
        UserFactory::createMany(10);

        $this->createClientWithCredentials()->request('GET', '/api/users');
        $this->assertResponseStatusCodeSame(Response::HTTP_OK);
        $this->assertResponseHeaderSame('content-type', 'application/ld+json; charset=utf-8');
        $this->assertJsonContains([
            '@context' => '/api/contexts/User',
            '@id' => '/api/users',
            '@type' => 'hydra:Collection',
            'hydra:totalItems' => 11,
        ]);
    }

    /**
     * @throws RedirectionExceptionInterface
     * @throws DecodingExceptionInterface
     * @throws ClientExceptionInterface
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     */
    public function testCreate(): void
    {
        UserFactory::createOne(['username' => self::USER_ADMIN]);
        $this->createClientWithCredentials()->request('POST', '/api/users',['json' => [
            'username' => self::USER_CREATE,
            'password' => '12345678',
            'enabled' => true,
        ]]);

        $this->assertResponseStatusCodeSame(201);
        $this->assertResponseHeaderSame('content-type', 'application/ld+json; charset=utf-8');
        $this->assertJsonContains([
            '@context' => '/api/contexts/UserOutput',
            '@type' => 'User',
            'username' => self::USER_CREATE,
            'enabled' => true,
        ]);
    }

    /**
     * @throws RedirectionExceptionInterface
     * @throws DecodingExceptionInterface
     * @throws ClientExceptionInterface
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     */
    public function testUpdateBook(): void
    {
        UserFactory::createOne(['username' => self::USER_ADMIN]);
        UserFactory::createOne(['username' => self::USER_UPDATE]);

        $iri = $this->findIriBy(User::class, ['username' => self::USER_UPDATE]);

        $this->createClientWithCredentials()->request('PATCH', $iri, ['json' => [
            'username' => self::USER_UPDATE,
        ]]);

        $this->assertResponseIsSuccessful();
        $this->assertJsonContains([
            '@id' => $iri,
            'username' => self::USER_UPDATE,
        ]);
    }

    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws DecodingExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function testDeleteBook(): void
    {
        UserFactory::createOne(['username' => self::USER_ADMIN]);
        UserFactory::createOne(['username' => self::USER_DELETE]);

        $iri = $this->findIriBy(User::class, ['username' => self::USER_DELETE]);

        $this->createClientWithCredentials()->request('DELETE', $iri);
        $this->assertResponseStatusCodeSame(204);
        $this->assertNull(
            static::getContainer()->get('doctrine')->getRepository(User::class)->findOneBy(['username' => self::USER_DELETE])
        );
    }
}